/*
** ADM-XRC Series Driver
**
** (C) Copyright 2001-2004 Alpha Data
**
** configrw.c - IOCTL handlers for getting and setting the local bus
**              space configurations.
**
** ----------------------------------------------------------------------------
*/

#include "platform.h"
#include "ioctls.h"
#include "device.h"
#include "configrw.h"

OS_STATUS
ADMXRC_ReadConfig(
  DeviceContext* pDevCtx,
  OS_IO_REQUEST* pReq,
  ADMXRC_IOCTLS_READCONFIG* pIoctl,
  unsigned long inSize,
  unsigned long outSize)
{
  os_uint32 val32;
  unsigned long index;
  MethodResult res;
  OS_STATUS status;

  if (pIoctl == NULL || inSize != sizeof(pIoctl->in) || outSize != sizeof(pIoctl->out)) {
    return OS_MAKE_STATUS(ADMXRC2_INVALID_PARAMETER);
  }

  index = pIoctl->in.index;
  res = pDevCtx->methods.configMem(pDevCtx, FALSE, index, &val32);
  if (res == MethodResult_Success) {
    status = OS_STATUS_SUCCESS;
    pIoctl->out.value = val32;
    pReq->status.length = sizeof(pIoctl->out);
  } else {
    if (res == MethodResult_InvalidLocation) {
      status = OS_MAKE_STATUS(ADMXRC2_INVALID_PARAMETER);
    } else if (res == MethodResult_Timeout) {
      status = OS_MAKE_STATUS(ADMXRC2_FAILED);
    } else {
      status = OS_MAKE_STATUS(ADMXRC2_UNKNOWN_ERROR);
    }
  }

  return status;
}

OS_STATUS
ADMXRC_WriteConfig(
  DeviceContext* pDevCtx,
  OS_IO_REQUEST* pReq,
  ADMXRC_IOCTLS_WRITECONFIG* pIoctl,
  unsigned long inSize)
{
  unsigned long              index;
  os_uint32                  val32;
  MethodResult               res;
  OS_STATUS                  status;

  if (pIoctl == NULL || inSize != sizeof(pIoctl->in)) {
    return OS_MAKE_STATUS(ADMXRC2_INVALID_PARAMETER);
  }

  if (pIoctl->in.signature1 != 0x587d3f31 ||
      pIoctl->in.signature2 != 0x043a749f) {
    return OS_MAKE_STATUS(ADMXRC2_INVALID_PARAMETER);
  }

  index = pIoctl->in.index;
  val32 = pIoctl->in.value;
  res = pDevCtx->methods.configMem(pDevCtx, TRUE, index, &val32);
  if (res == MethodResult_Success) {
    status = OS_STATUS_SUCCESS;
  } else {
    if (res == MethodResult_InvalidLocation) {
      status = OS_MAKE_STATUS(ADMXRC2_INVALID_PARAMETER);
    } else if (res == MethodResult_Timeout) {
      status = OS_MAKE_STATUS(ADMXRC2_FAILED);
    } else {
      status = OS_MAKE_STATUS(ADMXRC2_UNKNOWN_ERROR);
    }
  }

  return OS_STATUS_SUCCESS;
}

